

#include "raytracer.h"
#include <math.h>
#include <float.h>
#include <limits.h>


template <class S, template <typename> class I >  RayTracer<S,I>::RayTracer()
: integrator_(), integrator(integrator_), scene(integrator_.scene) {
}

template <class S, template <typename> class I >  RayTracer<S,I>::~RayTracer() {
}

template <class S, template <typename> class I >  void RayTracer<S,I>::trace( Surface<ColorFRGB> &target ) const {
    const unsigned int width  = target.getWidth();
    const unsigned int height = target.getHeight();

    ColorFRGB * const surface = target.checkOut();

    static bool FIRSTRUN = true;
    int W = FIRSTRUN ? 1 : 4;
    static int X = 0;
    static int Y = 0;
    FIRSTRUN = false;

    const float u_step = static_cast<float>(W) *    1.0f / static_cast<float>(width);
    const float v_step = static_cast<float>(W) *   -1.0f / static_cast<float>(height);
    float u=0.0f,v=1.0f;

    for( unsigned int y=Y; y<height; y+=W ){
        u = 0.0f;
        for( unsigned int x=X; x<width; x+=W ){
            surface[y*width+x] = integrator.integrate(u, v);
            u += u_step;
        }
        v += v_step;
    }
    target.checkIn();

    X++;
    if( X >= W ){
        Y++;
        X = 0;
    }
    if( Y >= W ){
        Y=X=0;
    }
}


